# pylint:disable=line-too-long
import logging

from ...sim_type import SimTypeFunction,     SimTypeShort, SimTypeInt, SimTypeLong, SimTypeLongLong, SimTypeDouble, SimTypeFloat,     SimTypePointer,     SimTypeChar,     SimStruct,     SimTypeFixedSizeArray,     SimTypeBottom,     SimUnion,     SimTypeBool
from ...calling_conventions import SimCCStdcall, SimCCMicrosoftAMD64
from .. import SIM_PROCEDURES as P
from . import SimLibrary


_l = logging.getLogger(name=__name__)


lib = SimLibrary()
lib.set_default_cc('X86', SimCCStdcall)
lib.set_default_cc('AMD64', SimCCMicrosoftAMD64)
lib.set_library_names("rasapi32.dll")
prototypes = \
    {
        #
        'RasDialA': SimTypeFunction([SimTypePointer(SimStruct({"dwSize": SimTypeInt(signed=False, label="UInt32"), "dwfOptions": SimTypeInt(signed=False, label="UInt32"), "hwndParent": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "reserved": SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), "reserved1": SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), "RasEapInfo": SimStruct({"dwSizeofEapInfo": SimTypeInt(signed=False, label="UInt32"), "pbEapInfo": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="RASEAPINFO", pack=False, align=None), "fSkipPppAuth": SimTypeInt(signed=True, label="Int32"), "RasDevSpecificInfo": SimStruct({"dwSize": SimTypeInt(signed=False, label="UInt32"), "pbDevSpecificInfo": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="RASDEVSPECIFICINFO", pack=False, align=None)}, name="RASDIALEXTENSIONS", pack=False, align=None), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimStruct({"dwSize": SimTypeInt(signed=False, label="UInt32"), "szEntryName": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 257), "szPhoneNumber": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 129), "szCallbackNumber": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 129), "szUserName": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 257), "szPassword": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 257), "szDomain": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 16), "dwSubEntry": SimTypeInt(signed=False, label="UInt32"), "dwCallbackId": SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), "dwIfIndex": SimTypeInt(signed=False, label="UInt32"), "szEncPassword": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="RASDIALPARAMSA", pack=False, align=None), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2", "param3", "param4", "param5"]),
        #
        'RasDialW': SimTypeFunction([SimTypePointer(SimStruct({"dwSize": SimTypeInt(signed=False, label="UInt32"), "dwfOptions": SimTypeInt(signed=False, label="UInt32"), "hwndParent": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "reserved": SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), "reserved1": SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), "RasEapInfo": SimStruct({"dwSizeofEapInfo": SimTypeInt(signed=False, label="UInt32"), "pbEapInfo": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="RASEAPINFO", pack=False, align=None), "fSkipPppAuth": SimTypeInt(signed=True, label="Int32"), "RasDevSpecificInfo": SimStruct({"dwSize": SimTypeInt(signed=False, label="UInt32"), "pbDevSpecificInfo": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="RASDEVSPECIFICINFO", pack=False, align=None)}, name="RASDIALEXTENSIONS", pack=False, align=None), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimStruct({"dwSize": SimTypeInt(signed=False, label="UInt32"), "szEntryName": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 257), "szPhoneNumber": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 129), "szCallbackNumber": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 129), "szUserName": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 257), "szPassword": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 257), "szDomain": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 16), "dwSubEntry": SimTypeInt(signed=False, label="UInt32"), "dwCallbackId": SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), "dwIfIndex": SimTypeInt(signed=False, label="UInt32"), "szEncPassword": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="RASDIALPARAMSW", pack=False, align=None), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2", "param3", "param4", "param5"]),
        #
        'RasEnumConnectionsA': SimTypeFunction([SimTypePointer(SimStruct({"dwSize": SimTypeInt(signed=False, label="UInt32"), "hrasconn": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "szEntryName": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 257), "szDeviceType": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 17), "szDeviceName": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 129), "szPhonebook": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 260), "dwSubEntry": SimTypeInt(signed=False, label="UInt32"), "guidEntry": SimTypeBottom(label="Guid"), "dwFlags": SimTypeInt(signed=False, label="UInt32"), "luid": SimTypeBottom(label="LUID"), "guidCorrelationId": SimTypeBottom(label="Guid")}, name="RASCONNA", pack=False, align=None), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2"]),
        #
        'RasEnumConnectionsW': SimTypeFunction([SimTypePointer(SimStruct({"dwSize": SimTypeInt(signed=False, label="UInt32"), "hrasconn": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "szEntryName": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 257), "szDeviceType": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 17), "szDeviceName": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 129), "szPhonebook": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 260), "dwSubEntry": SimTypeInt(signed=False, label="UInt32"), "guidEntry": SimTypeBottom(label="Guid"), "dwFlags": SimTypeInt(signed=False, label="UInt32"), "luid": SimTypeBottom(label="LUID"), "guidCorrelationId": SimTypeBottom(label="Guid")}, name="RASCONNW", pack=False, align=None), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2"]),
        #
        'RasEnumEntriesA': SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimStruct({"dwSize": SimTypeInt(signed=False, label="UInt32"), "szEntryName": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 257), "dwFlags": SimTypeInt(signed=False, label="UInt32"), "szPhonebookPath": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 261)}, name="RASENTRYNAMEA", pack=False, align=None), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2", "param3", "param4"]),
        #
        'RasEnumEntriesW': SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimStruct({"dwSize": SimTypeInt(signed=False, label="UInt32"), "szEntryName": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 257), "dwFlags": SimTypeInt(signed=False, label="UInt32"), "szPhonebookPath": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 261)}, name="RASENTRYNAMEW", pack=False, align=None), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2", "param3", "param4"]),
        #
        'RasGetConnectStatusA': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimStruct({"dwSize": SimTypeInt(signed=False, label="UInt32"), "rasconnstate": SimTypeInt(signed=False, label="RASCONNSTATE"), "dwError": SimTypeInt(signed=False, label="UInt32"), "szDeviceType": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 17), "szDeviceName": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 129), "szPhoneNumber": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 129), "localEndPoint": SimStruct({"dwType": SimTypeInt(signed=False, label="UInt32"), "Anonymous": SimUnion({"ipv4": SimStruct({"S_un": SimUnion({"S_un_b": SimStruct({"s_b1": SimTypeChar(label="Byte"), "s_b2": SimTypeChar(label="Byte"), "s_b3": SimTypeChar(label="Byte"), "s_b4": SimTypeChar(label="Byte")}, name="_S_un_b_e__Struct", pack=False, align=None), "S_un_w": SimStruct({"s_w1": SimTypeShort(signed=False, label="UInt16"), "s_w2": SimTypeShort(signed=False, label="UInt16")}, name="_S_un_w_e__Struct", pack=False, align=None), "S_addr": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")}, name="IN_ADDR", pack=False, align=None), "ipv6": SimStruct({"u": SimUnion({"Byte": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "Word": SimTypeFixedSizeArray(SimTypeShort(signed=False, label="UInt16"), 8)}, name="<anon>", label="None")}, name="IN6_ADDR", pack=False, align=None)}, name="<anon>", label="None")}, name="RASTUNNELENDPOINT", pack=False, align=None), "remoteEndPoint": SimStruct({"dwType": SimTypeInt(signed=False, label="UInt32"), "Anonymous": SimUnion({"ipv4": SimStruct({"S_un": SimUnion({"S_un_b": SimStruct({"s_b1": SimTypeChar(label="Byte"), "s_b2": SimTypeChar(label="Byte"), "s_b3": SimTypeChar(label="Byte"), "s_b4": SimTypeChar(label="Byte")}, name="_S_un_b_e__Struct", pack=False, align=None), "S_un_w": SimStruct({"s_w1": SimTypeShort(signed=False, label="UInt16"), "s_w2": SimTypeShort(signed=False, label="UInt16")}, name="_S_un_w_e__Struct", pack=False, align=None), "S_addr": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")}, name="IN_ADDR", pack=False, align=None), "ipv6": SimStruct({"u": SimUnion({"Byte": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "Word": SimTypeFixedSizeArray(SimTypeShort(signed=False, label="UInt16"), 8)}, name="<anon>", label="None")}, name="IN6_ADDR", pack=False, align=None)}, name="<anon>", label="None")}, name="RASTUNNELENDPOINT", pack=False, align=None), "rasconnsubstate": SimTypeInt(signed=False, label="RASCONNSUBSTATE")}, name="RASCONNSTATUSA", pack=False, align=None), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1"]),
        #
        'RasGetConnectStatusW': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimStruct({"dwSize": SimTypeInt(signed=False, label="UInt32"), "rasconnstate": SimTypeInt(signed=False, label="RASCONNSTATE"), "dwError": SimTypeInt(signed=False, label="UInt32"), "szDeviceType": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 17), "szDeviceName": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 129), "szPhoneNumber": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 129), "localEndPoint": SimStruct({"dwType": SimTypeInt(signed=False, label="UInt32"), "Anonymous": SimUnion({"ipv4": SimStruct({"S_un": SimUnion({"S_un_b": SimStruct({"s_b1": SimTypeChar(label="Byte"), "s_b2": SimTypeChar(label="Byte"), "s_b3": SimTypeChar(label="Byte"), "s_b4": SimTypeChar(label="Byte")}, name="_S_un_b_e__Struct", pack=False, align=None), "S_un_w": SimStruct({"s_w1": SimTypeShort(signed=False, label="UInt16"), "s_w2": SimTypeShort(signed=False, label="UInt16")}, name="_S_un_w_e__Struct", pack=False, align=None), "S_addr": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")}, name="IN_ADDR", pack=False, align=None), "ipv6": SimStruct({"u": SimUnion({"Byte": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "Word": SimTypeFixedSizeArray(SimTypeShort(signed=False, label="UInt16"), 8)}, name="<anon>", label="None")}, name="IN6_ADDR", pack=False, align=None)}, name="<anon>", label="None")}, name="RASTUNNELENDPOINT", pack=False, align=None), "remoteEndPoint": SimStruct({"dwType": SimTypeInt(signed=False, label="UInt32"), "Anonymous": SimUnion({"ipv4": SimStruct({"S_un": SimUnion({"S_un_b": SimStruct({"s_b1": SimTypeChar(label="Byte"), "s_b2": SimTypeChar(label="Byte"), "s_b3": SimTypeChar(label="Byte"), "s_b4": SimTypeChar(label="Byte")}, name="_S_un_b_e__Struct", pack=False, align=None), "S_un_w": SimStruct({"s_w1": SimTypeShort(signed=False, label="UInt16"), "s_w2": SimTypeShort(signed=False, label="UInt16")}, name="_S_un_w_e__Struct", pack=False, align=None), "S_addr": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")}, name="IN_ADDR", pack=False, align=None), "ipv6": SimStruct({"u": SimUnion({"Byte": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "Word": SimTypeFixedSizeArray(SimTypeShort(signed=False, label="UInt16"), 8)}, name="<anon>", label="None")}, name="IN6_ADDR", pack=False, align=None)}, name="<anon>", label="None")}, name="RASTUNNELENDPOINT", pack=False, align=None), "rasconnsubstate": SimTypeInt(signed=False, label="RASCONNSUBSTATE")}, name="RASCONNSTATUSW", pack=False, align=None), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1"]),
        #
        'RasGetErrorStringA': SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), label="LPArray", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["ResourceId", "lpszString", "InBufSize"]),
        #
        'RasGetErrorStringW': SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Char"), label="LPArray", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["ResourceId", "lpszString", "InBufSize"]),
        #
        'RasHangUpA': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]),
        #
        'RasHangUpW': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]),
        #
        'RasGetProjectionInfoA': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="RASPROJECTION"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2", "param3"]),
        #
        'RasGetProjectionInfoW': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="RASPROJECTION"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2", "param3"]),
        #
        'RasCreatePhonebookEntryA': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1"]),
        #
        'RasCreatePhonebookEntryW': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1"]),
        #
        'RasEditPhonebookEntryA': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2"]),
        #
        'RasEditPhonebookEntryW': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2"]),
        #
        'RasSetEntryDialParamsA': SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimStruct({"dwSize": SimTypeInt(signed=False, label="UInt32"), "szEntryName": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 257), "szPhoneNumber": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 129), "szCallbackNumber": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 129), "szUserName": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 257), "szPassword": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 257), "szDomain": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 16), "dwSubEntry": SimTypeInt(signed=False, label="UInt32"), "dwCallbackId": SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), "dwIfIndex": SimTypeInt(signed=False, label="UInt32"), "szEncPassword": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="RASDIALPARAMSA", pack=False, align=None), offset=0), SimTypeInt(signed=True, label="Int32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2"]),
        #
        'RasSetEntryDialParamsW': SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimStruct({"dwSize": SimTypeInt(signed=False, label="UInt32"), "szEntryName": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 257), "szPhoneNumber": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 129), "szCallbackNumber": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 129), "szUserName": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 257), "szPassword": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 257), "szDomain": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 16), "dwSubEntry": SimTypeInt(signed=False, label="UInt32"), "dwCallbackId": SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), "dwIfIndex": SimTypeInt(signed=False, label="UInt32"), "szEncPassword": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="RASDIALPARAMSW", pack=False, align=None), offset=0), SimTypeInt(signed=True, label="Int32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2"]),
        #
        'RasGetEntryDialParamsA': SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimStruct({"dwSize": SimTypeInt(signed=False, label="UInt32"), "szEntryName": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 257), "szPhoneNumber": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 129), "szCallbackNumber": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 129), "szUserName": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 257), "szPassword": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 257), "szDomain": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 16), "dwSubEntry": SimTypeInt(signed=False, label="UInt32"), "dwCallbackId": SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), "dwIfIndex": SimTypeInt(signed=False, label="UInt32"), "szEncPassword": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="RASDIALPARAMSA", pack=False, align=None), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2"]),
        #
        'RasGetEntryDialParamsW': SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimStruct({"dwSize": SimTypeInt(signed=False, label="UInt32"), "szEntryName": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 257), "szPhoneNumber": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 129), "szCallbackNumber": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 129), "szUserName": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 257), "szPassword": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 257), "szDomain": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 16), "dwSubEntry": SimTypeInt(signed=False, label="UInt32"), "dwCallbackId": SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), "dwIfIndex": SimTypeInt(signed=False, label="UInt32"), "szEncPassword": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="RASDIALPARAMSW", pack=False, align=None), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2"]),
        #
        'RasEnumDevicesA': SimTypeFunction([SimTypePointer(SimStruct({"dwSize": SimTypeInt(signed=False, label="UInt32"), "szDeviceType": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 17), "szDeviceName": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 129)}, name="RASDEVINFOA", pack=False, align=None), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2"]),
        #
        'RasEnumDevicesW': SimTypeFunction([SimTypePointer(SimStruct({"dwSize": SimTypeInt(signed=False, label="UInt32"), "szDeviceType": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 17), "szDeviceName": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 129)}, name="RASDEVINFOW", pack=False, align=None), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2"]),
        #
        'RasGetCountryInfoA': SimTypeFunction([SimTypePointer(SimStruct({"dwSize": SimTypeInt(signed=False, label="UInt32"), "dwCountryID": SimTypeInt(signed=False, label="UInt32"), "dwNextCountryID": SimTypeInt(signed=False, label="UInt32"), "dwCountryCode": SimTypeInt(signed=False, label="UInt32"), "dwCountryNameOffset": SimTypeInt(signed=False, label="UInt32")}, name="RASCTRYINFO", pack=False, align=None), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1"]),
        #
        'RasGetCountryInfoW': SimTypeFunction([SimTypePointer(SimStruct({"dwSize": SimTypeInt(signed=False, label="UInt32"), "dwCountryID": SimTypeInt(signed=False, label="UInt32"), "dwNextCountryID": SimTypeInt(signed=False, label="UInt32"), "dwCountryCode": SimTypeInt(signed=False, label="UInt32"), "dwCountryNameOffset": SimTypeInt(signed=False, label="UInt32")}, name="RASCTRYINFO", pack=False, align=None), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1"]),
        #
        'RasGetEntryPropertiesA': SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimStruct({"dwSize": SimTypeInt(signed=False, label="UInt32"), "dwfOptions": SimTypeInt(signed=False, label="UInt32"), "dwCountryID": SimTypeInt(signed=False, label="UInt32"), "dwCountryCode": SimTypeInt(signed=False, label="UInt32"), "szAreaCode": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 11), "szLocalPhoneNumber": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 129), "dwAlternateOffset": SimTypeInt(signed=False, label="UInt32"), "ipaddr": SimStruct({"a": SimTypeChar(label="Byte"), "b": SimTypeChar(label="Byte"), "c": SimTypeChar(label="Byte"), "d": SimTypeChar(label="Byte")}, name="RASIPADDR", pack=False, align=None), "ipaddrDns": SimStruct({"a": SimTypeChar(label="Byte"), "b": SimTypeChar(label="Byte"), "c": SimTypeChar(label="Byte"), "d": SimTypeChar(label="Byte")}, name="RASIPADDR", pack=False, align=None), "ipaddrDnsAlt": SimStruct({"a": SimTypeChar(label="Byte"), "b": SimTypeChar(label="Byte"), "c": SimTypeChar(label="Byte"), "d": SimTypeChar(label="Byte")}, name="RASIPADDR", pack=False, align=None), "ipaddrWins": SimStruct({"a": SimTypeChar(label="Byte"), "b": SimTypeChar(label="Byte"), "c": SimTypeChar(label="Byte"), "d": SimTypeChar(label="Byte")}, name="RASIPADDR", pack=False, align=None), "ipaddrWinsAlt": SimStruct({"a": SimTypeChar(label="Byte"), "b": SimTypeChar(label="Byte"), "c": SimTypeChar(label="Byte"), "d": SimTypeChar(label="Byte")}, name="RASIPADDR", pack=False, align=None), "dwFrameSize": SimTypeInt(signed=False, label="UInt32"), "dwfNetProtocols": SimTypeInt(signed=False, label="UInt32"), "dwFramingProtocol": SimTypeInt(signed=False, label="UInt32"), "szScript": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 260), "szAutodialDll": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 260), "szAutodialFunc": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 260), "szDeviceType": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 17), "szDeviceName": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 129), "szX25PadType": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 33), "szX25Address": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 201), "szX25Facilities": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 201), "szX25UserData": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 201), "dwChannels": SimTypeInt(signed=False, label="UInt32"), "dwReserved1": SimTypeInt(signed=False, label="UInt32"), "dwReserved2": SimTypeInt(signed=False, label="UInt32"), "dwSubEntries": SimTypeInt(signed=False, label="UInt32"), "dwDialMode": SimTypeInt(signed=False, label="RASENTRY_DIAL_MODE"), "dwDialExtraPercent": SimTypeInt(signed=False, label="UInt32"), "dwDialExtraSampleSeconds": SimTypeInt(signed=False, label="UInt32"), "dwHangUpExtraPercent": SimTypeInt(signed=False, label="UInt32"), "dwHangUpExtraSampleSeconds": SimTypeInt(signed=False, label="UInt32"), "dwIdleDisconnectSeconds": SimTypeInt(signed=False, label="UInt32"), "dwType": SimTypeInt(signed=False, label="UInt32"), "dwEncryptionType": SimTypeInt(signed=False, label="UInt32"), "dwCustomAuthKey": SimTypeInt(signed=False, label="UInt32"), "guidId": SimTypeBottom(label="Guid"), "szCustomDialDll": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 260), "dwVpnStrategy": SimTypeInt(signed=False, label="UInt32"), "dwfOptions2": SimTypeInt(signed=False, label="UInt32"), "dwfOptions3": SimTypeInt(signed=False, label="UInt32"), "szDnsSuffix": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 256), "dwTcpWindowSize": SimTypeInt(signed=False, label="UInt32"), "szPrerequisitePbk": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 260), "szPrerequisiteEntry": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 257), "dwRedialCount": SimTypeInt(signed=False, label="UInt32"), "dwRedialPause": SimTypeInt(signed=False, label="UInt32"), "ipv6addrDns": SimStruct({"u": SimUnion({"Byte": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "Word": SimTypeFixedSizeArray(SimTypeShort(signed=False, label="UInt16"), 8)}, name="<anon>", label="None")}, name="IN6_ADDR", pack=False, align=None), "ipv6addrDnsAlt": SimStruct({"u": SimUnion({"Byte": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "Word": SimTypeFixedSizeArray(SimTypeShort(signed=False, label="UInt16"), 8)}, name="<anon>", label="None")}, name="IN6_ADDR", pack=False, align=None), "dwIPv4InterfaceMetric": SimTypeInt(signed=False, label="UInt32"), "dwIPv6InterfaceMetric": SimTypeInt(signed=False, label="UInt32"), "ipv6addr": SimStruct({"u": SimUnion({"Byte": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "Word": SimTypeFixedSizeArray(SimTypeShort(signed=False, label="UInt16"), 8)}, name="<anon>", label="None")}, name="IN6_ADDR", pack=False, align=None), "dwIPv6PrefixLength": SimTypeInt(signed=False, label="UInt32"), "dwNetworkOutageTime": SimTypeInt(signed=False, label="UInt32"), "szIDi": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 257), "szIDr": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 257), "fIsImsConfig": SimTypeInt(signed=True, label="Int32"), "IdiType": SimTypeInt(signed=False, label="IKEV2_ID_PAYLOAD_TYPE"), "IdrType": SimTypeInt(signed=False, label="IKEV2_ID_PAYLOAD_TYPE"), "fDisableIKEv2Fragmentation": SimTypeInt(signed=True, label="Int32")}, name="RASENTRYA", pack=False, align=None), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2", "param3", "param4", "param5"]),
        #
        'RasGetEntryPropertiesW': SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimStruct({"dwSize": SimTypeInt(signed=False, label="UInt32"), "dwfOptions": SimTypeInt(signed=False, label="UInt32"), "dwCountryID": SimTypeInt(signed=False, label="UInt32"), "dwCountryCode": SimTypeInt(signed=False, label="UInt32"), "szAreaCode": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 11), "szLocalPhoneNumber": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 129), "dwAlternateOffset": SimTypeInt(signed=False, label="UInt32"), "ipaddr": SimStruct({"a": SimTypeChar(label="Byte"), "b": SimTypeChar(label="Byte"), "c": SimTypeChar(label="Byte"), "d": SimTypeChar(label="Byte")}, name="RASIPADDR", pack=False, align=None), "ipaddrDns": SimStruct({"a": SimTypeChar(label="Byte"), "b": SimTypeChar(label="Byte"), "c": SimTypeChar(label="Byte"), "d": SimTypeChar(label="Byte")}, name="RASIPADDR", pack=False, align=None), "ipaddrDnsAlt": SimStruct({"a": SimTypeChar(label="Byte"), "b": SimTypeChar(label="Byte"), "c": SimTypeChar(label="Byte"), "d": SimTypeChar(label="Byte")}, name="RASIPADDR", pack=False, align=None), "ipaddrWins": SimStruct({"a": SimTypeChar(label="Byte"), "b": SimTypeChar(label="Byte"), "c": SimTypeChar(label="Byte"), "d": SimTypeChar(label="Byte")}, name="RASIPADDR", pack=False, align=None), "ipaddrWinsAlt": SimStruct({"a": SimTypeChar(label="Byte"), "b": SimTypeChar(label="Byte"), "c": SimTypeChar(label="Byte"), "d": SimTypeChar(label="Byte")}, name="RASIPADDR", pack=False, align=None), "dwFrameSize": SimTypeInt(signed=False, label="UInt32"), "dwfNetProtocols": SimTypeInt(signed=False, label="UInt32"), "dwFramingProtocol": SimTypeInt(signed=False, label="UInt32"), "szScript": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 260), "szAutodialDll": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 260), "szAutodialFunc": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 260), "szDeviceType": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 17), "szDeviceName": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 129), "szX25PadType": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 33), "szX25Address": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 201), "szX25Facilities": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 201), "szX25UserData": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 201), "dwChannels": SimTypeInt(signed=False, label="UInt32"), "dwReserved1": SimTypeInt(signed=False, label="UInt32"), "dwReserved2": SimTypeInt(signed=False, label="UInt32"), "dwSubEntries": SimTypeInt(signed=False, label="UInt32"), "dwDialMode": SimTypeInt(signed=False, label="RASENTRY_DIAL_MODE"), "dwDialExtraPercent": SimTypeInt(signed=False, label="UInt32"), "dwDialExtraSampleSeconds": SimTypeInt(signed=False, label="UInt32"), "dwHangUpExtraPercent": SimTypeInt(signed=False, label="UInt32"), "dwHangUpExtraSampleSeconds": SimTypeInt(signed=False, label="UInt32"), "dwIdleDisconnectSeconds": SimTypeInt(signed=False, label="UInt32"), "dwType": SimTypeInt(signed=False, label="UInt32"), "dwEncryptionType": SimTypeInt(signed=False, label="UInt32"), "dwCustomAuthKey": SimTypeInt(signed=False, label="UInt32"), "guidId": SimTypeBottom(label="Guid"), "szCustomDialDll": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 260), "dwVpnStrategy": SimTypeInt(signed=False, label="UInt32"), "dwfOptions2": SimTypeInt(signed=False, label="UInt32"), "dwfOptions3": SimTypeInt(signed=False, label="UInt32"), "szDnsSuffix": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 256), "dwTcpWindowSize": SimTypeInt(signed=False, label="UInt32"), "szPrerequisitePbk": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 260), "szPrerequisiteEntry": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 257), "dwRedialCount": SimTypeInt(signed=False, label="UInt32"), "dwRedialPause": SimTypeInt(signed=False, label="UInt32"), "ipv6addrDns": SimStruct({"u": SimUnion({"Byte": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "Word": SimTypeFixedSizeArray(SimTypeShort(signed=False, label="UInt16"), 8)}, name="<anon>", label="None")}, name="IN6_ADDR", pack=False, align=None), "ipv6addrDnsAlt": SimStruct({"u": SimUnion({"Byte": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "Word": SimTypeFixedSizeArray(SimTypeShort(signed=False, label="UInt16"), 8)}, name="<anon>", label="None")}, name="IN6_ADDR", pack=False, align=None), "dwIPv4InterfaceMetric": SimTypeInt(signed=False, label="UInt32"), "dwIPv6InterfaceMetric": SimTypeInt(signed=False, label="UInt32"), "ipv6addr": SimStruct({"u": SimUnion({"Byte": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "Word": SimTypeFixedSizeArray(SimTypeShort(signed=False, label="UInt16"), 8)}, name="<anon>", label="None")}, name="IN6_ADDR", pack=False, align=None), "dwIPv6PrefixLength": SimTypeInt(signed=False, label="UInt32"), "dwNetworkOutageTime": SimTypeInt(signed=False, label="UInt32"), "szIDi": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 257), "szIDr": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 257), "fIsImsConfig": SimTypeInt(signed=True, label="Int32"), "IdiType": SimTypeInt(signed=False, label="IKEV2_ID_PAYLOAD_TYPE"), "IdrType": SimTypeInt(signed=False, label="IKEV2_ID_PAYLOAD_TYPE"), "fDisableIKEv2Fragmentation": SimTypeInt(signed=True, label="Int32")}, name="RASENTRYW", pack=False, align=None), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2", "param3", "param4", "param5"]),
        #
        'RasSetEntryPropertiesA': SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimStruct({"dwSize": SimTypeInt(signed=False, label="UInt32"), "dwfOptions": SimTypeInt(signed=False, label="UInt32"), "dwCountryID": SimTypeInt(signed=False, label="UInt32"), "dwCountryCode": SimTypeInt(signed=False, label="UInt32"), "szAreaCode": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 11), "szLocalPhoneNumber": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 129), "dwAlternateOffset": SimTypeInt(signed=False, label="UInt32"), "ipaddr": SimStruct({"a": SimTypeChar(label="Byte"), "b": SimTypeChar(label="Byte"), "c": SimTypeChar(label="Byte"), "d": SimTypeChar(label="Byte")}, name="RASIPADDR", pack=False, align=None), "ipaddrDns": SimStruct({"a": SimTypeChar(label="Byte"), "b": SimTypeChar(label="Byte"), "c": SimTypeChar(label="Byte"), "d": SimTypeChar(label="Byte")}, name="RASIPADDR", pack=False, align=None), "ipaddrDnsAlt": SimStruct({"a": SimTypeChar(label="Byte"), "b": SimTypeChar(label="Byte"), "c": SimTypeChar(label="Byte"), "d": SimTypeChar(label="Byte")}, name="RASIPADDR", pack=False, align=None), "ipaddrWins": SimStruct({"a": SimTypeChar(label="Byte"), "b": SimTypeChar(label="Byte"), "c": SimTypeChar(label="Byte"), "d": SimTypeChar(label="Byte")}, name="RASIPADDR", pack=False, align=None), "ipaddrWinsAlt": SimStruct({"a": SimTypeChar(label="Byte"), "b": SimTypeChar(label="Byte"), "c": SimTypeChar(label="Byte"), "d": SimTypeChar(label="Byte")}, name="RASIPADDR", pack=False, align=None), "dwFrameSize": SimTypeInt(signed=False, label="UInt32"), "dwfNetProtocols": SimTypeInt(signed=False, label="UInt32"), "dwFramingProtocol": SimTypeInt(signed=False, label="UInt32"), "szScript": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 260), "szAutodialDll": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 260), "szAutodialFunc": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 260), "szDeviceType": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 17), "szDeviceName": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 129), "szX25PadType": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 33), "szX25Address": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 201), "szX25Facilities": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 201), "szX25UserData": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 201), "dwChannels": SimTypeInt(signed=False, label="UInt32"), "dwReserved1": SimTypeInt(signed=False, label="UInt32"), "dwReserved2": SimTypeInt(signed=False, label="UInt32"), "dwSubEntries": SimTypeInt(signed=False, label="UInt32"), "dwDialMode": SimTypeInt(signed=False, label="RASENTRY_DIAL_MODE"), "dwDialExtraPercent": SimTypeInt(signed=False, label="UInt32"), "dwDialExtraSampleSeconds": SimTypeInt(signed=False, label="UInt32"), "dwHangUpExtraPercent": SimTypeInt(signed=False, label="UInt32"), "dwHangUpExtraSampleSeconds": SimTypeInt(signed=False, label="UInt32"), "dwIdleDisconnectSeconds": SimTypeInt(signed=False, label="UInt32"), "dwType": SimTypeInt(signed=False, label="UInt32"), "dwEncryptionType": SimTypeInt(signed=False, label="UInt32"), "dwCustomAuthKey": SimTypeInt(signed=False, label="UInt32"), "guidId": SimTypeBottom(label="Guid"), "szCustomDialDll": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 260), "dwVpnStrategy": SimTypeInt(signed=False, label="UInt32"), "dwfOptions2": SimTypeInt(signed=False, label="UInt32"), "dwfOptions3": SimTypeInt(signed=False, label="UInt32"), "szDnsSuffix": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 256), "dwTcpWindowSize": SimTypeInt(signed=False, label="UInt32"), "szPrerequisitePbk": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 260), "szPrerequisiteEntry": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 257), "dwRedialCount": SimTypeInt(signed=False, label="UInt32"), "dwRedialPause": SimTypeInt(signed=False, label="UInt32"), "ipv6addrDns": SimStruct({"u": SimUnion({"Byte": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "Word": SimTypeFixedSizeArray(SimTypeShort(signed=False, label="UInt16"), 8)}, name="<anon>", label="None")}, name="IN6_ADDR", pack=False, align=None), "ipv6addrDnsAlt": SimStruct({"u": SimUnion({"Byte": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "Word": SimTypeFixedSizeArray(SimTypeShort(signed=False, label="UInt16"), 8)}, name="<anon>", label="None")}, name="IN6_ADDR", pack=False, align=None), "dwIPv4InterfaceMetric": SimTypeInt(signed=False, label="UInt32"), "dwIPv6InterfaceMetric": SimTypeInt(signed=False, label="UInt32"), "ipv6addr": SimStruct({"u": SimUnion({"Byte": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "Word": SimTypeFixedSizeArray(SimTypeShort(signed=False, label="UInt16"), 8)}, name="<anon>", label="None")}, name="IN6_ADDR", pack=False, align=None), "dwIPv6PrefixLength": SimTypeInt(signed=False, label="UInt32"), "dwNetworkOutageTime": SimTypeInt(signed=False, label="UInt32"), "szIDi": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 257), "szIDr": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 257), "fIsImsConfig": SimTypeInt(signed=True, label="Int32"), "IdiType": SimTypeInt(signed=False, label="IKEV2_ID_PAYLOAD_TYPE"), "IdrType": SimTypeInt(signed=False, label="IKEV2_ID_PAYLOAD_TYPE"), "fDisableIKEv2Fragmentation": SimTypeInt(signed=True, label="Int32")}, name="RASENTRYA", pack=False, align=None), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2", "param3", "param4", "param5"]),
        #
        'RasSetEntryPropertiesW': SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimStruct({"dwSize": SimTypeInt(signed=False, label="UInt32"), "dwfOptions": SimTypeInt(signed=False, label="UInt32"), "dwCountryID": SimTypeInt(signed=False, label="UInt32"), "dwCountryCode": SimTypeInt(signed=False, label="UInt32"), "szAreaCode": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 11), "szLocalPhoneNumber": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 129), "dwAlternateOffset": SimTypeInt(signed=False, label="UInt32"), "ipaddr": SimStruct({"a": SimTypeChar(label="Byte"), "b": SimTypeChar(label="Byte"), "c": SimTypeChar(label="Byte"), "d": SimTypeChar(label="Byte")}, name="RASIPADDR", pack=False, align=None), "ipaddrDns": SimStruct({"a": SimTypeChar(label="Byte"), "b": SimTypeChar(label="Byte"), "c": SimTypeChar(label="Byte"), "d": SimTypeChar(label="Byte")}, name="RASIPADDR", pack=False, align=None), "ipaddrDnsAlt": SimStruct({"a": SimTypeChar(label="Byte"), "b": SimTypeChar(label="Byte"), "c": SimTypeChar(label="Byte"), "d": SimTypeChar(label="Byte")}, name="RASIPADDR", pack=False, align=None), "ipaddrWins": SimStruct({"a": SimTypeChar(label="Byte"), "b": SimTypeChar(label="Byte"), "c": SimTypeChar(label="Byte"), "d": SimTypeChar(label="Byte")}, name="RASIPADDR", pack=False, align=None), "ipaddrWinsAlt": SimStruct({"a": SimTypeChar(label="Byte"), "b": SimTypeChar(label="Byte"), "c": SimTypeChar(label="Byte"), "d": SimTypeChar(label="Byte")}, name="RASIPADDR", pack=False, align=None), "dwFrameSize": SimTypeInt(signed=False, label="UInt32"), "dwfNetProtocols": SimTypeInt(signed=False, label="UInt32"), "dwFramingProtocol": SimTypeInt(signed=False, label="UInt32"), "szScript": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 260), "szAutodialDll": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 260), "szAutodialFunc": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 260), "szDeviceType": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 17), "szDeviceName": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 129), "szX25PadType": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 33), "szX25Address": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 201), "szX25Facilities": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 201), "szX25UserData": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 201), "dwChannels": SimTypeInt(signed=False, label="UInt32"), "dwReserved1": SimTypeInt(signed=False, label="UInt32"), "dwReserved2": SimTypeInt(signed=False, label="UInt32"), "dwSubEntries": SimTypeInt(signed=False, label="UInt32"), "dwDialMode": SimTypeInt(signed=False, label="RASENTRY_DIAL_MODE"), "dwDialExtraPercent": SimTypeInt(signed=False, label="UInt32"), "dwDialExtraSampleSeconds": SimTypeInt(signed=False, label="UInt32"), "dwHangUpExtraPercent": SimTypeInt(signed=False, label="UInt32"), "dwHangUpExtraSampleSeconds": SimTypeInt(signed=False, label="UInt32"), "dwIdleDisconnectSeconds": SimTypeInt(signed=False, label="UInt32"), "dwType": SimTypeInt(signed=False, label="UInt32"), "dwEncryptionType": SimTypeInt(signed=False, label="UInt32"), "dwCustomAuthKey": SimTypeInt(signed=False, label="UInt32"), "guidId": SimTypeBottom(label="Guid"), "szCustomDialDll": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 260), "dwVpnStrategy": SimTypeInt(signed=False, label="UInt32"), "dwfOptions2": SimTypeInt(signed=False, label="UInt32"), "dwfOptions3": SimTypeInt(signed=False, label="UInt32"), "szDnsSuffix": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 256), "dwTcpWindowSize": SimTypeInt(signed=False, label="UInt32"), "szPrerequisitePbk": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 260), "szPrerequisiteEntry": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 257), "dwRedialCount": SimTypeInt(signed=False, label="UInt32"), "dwRedialPause": SimTypeInt(signed=False, label="UInt32"), "ipv6addrDns": SimStruct({"u": SimUnion({"Byte": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "Word": SimTypeFixedSizeArray(SimTypeShort(signed=False, label="UInt16"), 8)}, name="<anon>", label="None")}, name="IN6_ADDR", pack=False, align=None), "ipv6addrDnsAlt": SimStruct({"u": SimUnion({"Byte": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "Word": SimTypeFixedSizeArray(SimTypeShort(signed=False, label="UInt16"), 8)}, name="<anon>", label="None")}, name="IN6_ADDR", pack=False, align=None), "dwIPv4InterfaceMetric": SimTypeInt(signed=False, label="UInt32"), "dwIPv6InterfaceMetric": SimTypeInt(signed=False, label="UInt32"), "ipv6addr": SimStruct({"u": SimUnion({"Byte": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "Word": SimTypeFixedSizeArray(SimTypeShort(signed=False, label="UInt16"), 8)}, name="<anon>", label="None")}, name="IN6_ADDR", pack=False, align=None), "dwIPv6PrefixLength": SimTypeInt(signed=False, label="UInt32"), "dwNetworkOutageTime": SimTypeInt(signed=False, label="UInt32"), "szIDi": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 257), "szIDr": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 257), "fIsImsConfig": SimTypeInt(signed=True, label="Int32"), "IdiType": SimTypeInt(signed=False, label="IKEV2_ID_PAYLOAD_TYPE"), "IdrType": SimTypeInt(signed=False, label="IKEV2_ID_PAYLOAD_TYPE"), "fDisableIKEv2Fragmentation": SimTypeInt(signed=True, label="Int32")}, name="RASENTRYW", pack=False, align=None), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2", "param3", "param4", "param5"]),
        #
        'RasRenameEntryA': SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2"]),
        #
        'RasRenameEntryW': SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2"]),
        #
        'RasDeleteEntryA': SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1"]),
        #
        'RasDeleteEntryW': SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1"]),
        #
        'RasValidateEntryNameA': SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1"]),
        #
        'RasValidateEntryNameW': SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1"]),
        #
        'RasConnectionNotificationA': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2"]),
        #
        'RasConnectionNotificationW': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2"]),
        #
        'RasGetSubEntryHandleA': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2"]),
        #
        'RasGetSubEntryHandleW': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2"]),
        #
        'RasGetCredentialsA': SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimStruct({"dwSize": SimTypeInt(signed=False, label="UInt32"), "dwMask": SimTypeInt(signed=False, label="UInt32"), "szUserName": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 257), "szPassword": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 257), "szDomain": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 16)}, name="RASCREDENTIALSA", pack=False, align=None), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2"]),
        #
        'RasGetCredentialsW': SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimStruct({"dwSize": SimTypeInt(signed=False, label="UInt32"), "dwMask": SimTypeInt(signed=False, label="UInt32"), "szUserName": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 257), "szPassword": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 257), "szDomain": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 16)}, name="RASCREDENTIALSW", pack=False, align=None), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2"]),
        #
        'RasSetCredentialsA': SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimStruct({"dwSize": SimTypeInt(signed=False, label="UInt32"), "dwMask": SimTypeInt(signed=False, label="UInt32"), "szUserName": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 257), "szPassword": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 257), "szDomain": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 16)}, name="RASCREDENTIALSA", pack=False, align=None), offset=0), SimTypeInt(signed=True, label="Int32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2", "param3"]),
        #
        'RasSetCredentialsW': SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimStruct({"dwSize": SimTypeInt(signed=False, label="UInt32"), "dwMask": SimTypeInt(signed=False, label="UInt32"), "szUserName": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 257), "szPassword": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 257), "szDomain": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 16)}, name="RASCREDENTIALSW", pack=False, align=None), offset=0), SimTypeInt(signed=True, label="Int32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2", "param3"]),
        #
        'RasGetSubEntryPropertiesA': SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimStruct({"dwSize": SimTypeInt(signed=False, label="UInt32"), "dwfFlags": SimTypeInt(signed=False, label="UInt32"), "szDeviceType": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 17), "szDeviceName": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 129), "szLocalPhoneNumber": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 129), "dwAlternateOffset": SimTypeInt(signed=False, label="UInt32")}, name="RASSUBENTRYA", pack=False, align=None), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2", "param3", "param4", "param5", "param6"]),
        #
        'RasGetSubEntryPropertiesW': SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimStruct({"dwSize": SimTypeInt(signed=False, label="UInt32"), "dwfFlags": SimTypeInt(signed=False, label="UInt32"), "szDeviceType": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 17), "szDeviceName": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 129), "szLocalPhoneNumber": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 129), "dwAlternateOffset": SimTypeInt(signed=False, label="UInt32")}, name="RASSUBENTRYW", pack=False, align=None), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2", "param3", "param4", "param5", "param6"]),
        #
        'RasSetSubEntryPropertiesA': SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimStruct({"dwSize": SimTypeInt(signed=False, label="UInt32"), "dwfFlags": SimTypeInt(signed=False, label="UInt32"), "szDeviceType": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 17), "szDeviceName": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 129), "szLocalPhoneNumber": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 129), "dwAlternateOffset": SimTypeInt(signed=False, label="UInt32")}, name="RASSUBENTRYA", pack=False, align=None), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2", "param3", "param4", "param5", "param6"]),
        #
        'RasSetSubEntryPropertiesW': SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimStruct({"dwSize": SimTypeInt(signed=False, label="UInt32"), "dwfFlags": SimTypeInt(signed=False, label="UInt32"), "szDeviceType": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 17), "szDeviceName": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 129), "szLocalPhoneNumber": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 129), "dwAlternateOffset": SimTypeInt(signed=False, label="UInt32")}, name="RASSUBENTRYW", pack=False, align=None), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2", "param3", "param4", "param5", "param6"]),
        #
        'RasGetAutodialAddressA': SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimStruct({"dwSize": SimTypeInt(signed=False, label="UInt32"), "dwFlags": SimTypeInt(signed=False, label="UInt32"), "dwDialingLocation": SimTypeInt(signed=False, label="UInt32"), "szEntry": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 257)}, name="RASAUTODIALENTRYA", pack=False, align=None), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2", "param3", "param4"]),
        #
        'RasGetAutodialAddressW': SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimStruct({"dwSize": SimTypeInt(signed=False, label="UInt32"), "dwFlags": SimTypeInt(signed=False, label="UInt32"), "dwDialingLocation": SimTypeInt(signed=False, label="UInt32"), "szEntry": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 257)}, name="RASAUTODIALENTRYW", pack=False, align=None), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2", "param3", "param4"]),
        #
        'RasSetAutodialAddressA': SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimStruct({"dwSize": SimTypeInt(signed=False, label="UInt32"), "dwFlags": SimTypeInt(signed=False, label="UInt32"), "dwDialingLocation": SimTypeInt(signed=False, label="UInt32"), "szEntry": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 257)}, name="RASAUTODIALENTRYA", pack=False, align=None), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2", "param3", "param4"]),
        #
        'RasSetAutodialAddressW': SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimStruct({"dwSize": SimTypeInt(signed=False, label="UInt32"), "dwFlags": SimTypeInt(signed=False, label="UInt32"), "dwDialingLocation": SimTypeInt(signed=False, label="UInt32"), "szEntry": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 257)}, name="RASAUTODIALENTRYW", pack=False, align=None), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2", "param3", "param4"]),
        #
        'RasEnumAutodialAddressesA': SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["lppRasAutodialAddresses", "lpdwcbRasAutodialAddresses", "lpdwcRasAutodialAddresses"]),
        #
        'RasEnumAutodialAddressesW': SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["lppRasAutodialAddresses", "lpdwcbRasAutodialAddresses", "lpdwcRasAutodialAddresses"]),
        #
        'RasGetAutodialEnableA': SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1"]),
        #
        'RasGetAutodialEnableW': SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1"]),
        #
        'RasSetAutodialEnableA': SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=True, label="Int32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1"]),
        #
        'RasSetAutodialEnableW': SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=True, label="Int32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1"]),
        #
        'RasGetAutodialParamA': SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2"]),
        #
        'RasGetAutodialParamW': SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2"]),
        #
        'RasSetAutodialParamA': SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2"]),
        #
        'RasSetAutodialParamW': SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2"]),
        #
        'RasGetPCscf': SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["lpszPCscf"]),
        #
        'RasInvokeEapUI': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimStruct({"dwSize": SimTypeInt(signed=False, label="UInt32"), "dwfOptions": SimTypeInt(signed=False, label="UInt32"), "hwndParent": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "reserved": SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), "reserved1": SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), "RasEapInfo": SimStruct({"dwSizeofEapInfo": SimTypeInt(signed=False, label="UInt32"), "pbEapInfo": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="RASEAPINFO", pack=False, align=None), "fSkipPppAuth": SimTypeInt(signed=True, label="Int32"), "RasDevSpecificInfo": SimStruct({"dwSize": SimTypeInt(signed=False, label="UInt32"), "pbDevSpecificInfo": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="RASDEVSPECIFICINFO", pack=False, align=None)}, name="RASDIALEXTENSIONS", pack=False, align=None), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2", "param3"]),
        #
        'RasGetLinkStatistics': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimStruct({"dwSize": SimTypeInt(signed=False, label="UInt32"), "dwBytesXmited": SimTypeInt(signed=False, label="UInt32"), "dwBytesRcved": SimTypeInt(signed=False, label="UInt32"), "dwFramesXmited": SimTypeInt(signed=False, label="UInt32"), "dwFramesRcved": SimTypeInt(signed=False, label="UInt32"), "dwCrcErr": SimTypeInt(signed=False, label="UInt32"), "dwTimeoutErr": SimTypeInt(signed=False, label="UInt32"), "dwAlignmentErr": SimTypeInt(signed=False, label="UInt32"), "dwHardwareOverrunErr": SimTypeInt(signed=False, label="UInt32"), "dwFramingErr": SimTypeInt(signed=False, label="UInt32"), "dwBufferOverrunErr": SimTypeInt(signed=False, label="UInt32"), "dwCompressionRatioIn": SimTypeInt(signed=False, label="UInt32"), "dwCompressionRatioOut": SimTypeInt(signed=False, label="UInt32"), "dwBps": SimTypeInt(signed=False, label="UInt32"), "dwConnectDuration": SimTypeInt(signed=False, label="UInt32")}, name="RAS_STATS", pack=False, align=None), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hRasConn", "dwSubEntry", "lpStatistics"]),
        #
        'RasGetConnectionStatistics': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimStruct({"dwSize": SimTypeInt(signed=False, label="UInt32"), "dwBytesXmited": SimTypeInt(signed=False, label="UInt32"), "dwBytesRcved": SimTypeInt(signed=False, label="UInt32"), "dwFramesXmited": SimTypeInt(signed=False, label="UInt32"), "dwFramesRcved": SimTypeInt(signed=False, label="UInt32"), "dwCrcErr": SimTypeInt(signed=False, label="UInt32"), "dwTimeoutErr": SimTypeInt(signed=False, label="UInt32"), "dwAlignmentErr": SimTypeInt(signed=False, label="UInt32"), "dwHardwareOverrunErr": SimTypeInt(signed=False, label="UInt32"), "dwFramingErr": SimTypeInt(signed=False, label="UInt32"), "dwBufferOverrunErr": SimTypeInt(signed=False, label="UInt32"), "dwCompressionRatioIn": SimTypeInt(signed=False, label="UInt32"), "dwCompressionRatioOut": SimTypeInt(signed=False, label="UInt32"), "dwBps": SimTypeInt(signed=False, label="UInt32"), "dwConnectDuration": SimTypeInt(signed=False, label="UInt32")}, name="RAS_STATS", pack=False, align=None), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hRasConn", "lpStatistics"]),
        #
        'RasClearLinkStatistics': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["hRasConn", "dwSubEntry"]),
        #
        'RasClearConnectionStatistics': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hRasConn"]),
        #
        'RasGetEapUserDataA': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hToken", "pszPhonebook", "pszEntry", "pbEapData", "pdwSizeofEapData"]),
        #
        'RasGetEapUserDataW': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hToken", "pszPhonebook", "pszEntry", "pbEapData", "pdwSizeofEapData"]),
        #
        'RasSetEapUserDataA': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["hToken", "pszPhonebook", "pszEntry", "pbEapData", "dwSizeofEapData"]),
        #
        'RasSetEapUserDataW': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["hToken", "pszPhonebook", "pszEntry", "pbEapData", "dwSizeofEapData"]),
        #
        'RasGetCustomAuthDataA': SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["pszPhonebook", "pszEntry", "pbCustomAuthData", "pdwSizeofCustomAuthData"]),
        #
        'RasGetCustomAuthDataW': SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["pszPhonebook", "pszEntry", "pbCustomAuthData", "pdwSizeofCustomAuthData"]),
        #
        'RasSetCustomAuthDataA': SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["pszPhonebook", "pszEntry", "pbCustomAuthData", "dwSizeofCustomAuthData"]),
        #
        'RasSetCustomAuthDataW': SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["pszPhonebook", "pszEntry", "pbCustomAuthData", "dwSizeofCustomAuthData"]),
        #
        'RasGetEapUserIdentityW': SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypePointer(SimStruct({"szUserName": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 257), "dwSizeofEapInfo": SimTypeInt(signed=False, label="UInt32"), "pbEapInfo": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="RASEAPUSERIDENTITYW", pack=False, align=None), offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["pszPhonebook", "pszEntry", "dwFlags", "hwnd", "ppRasEapUserIdentity"]),
        #
        'RasGetEapUserIdentityA': SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypePointer(SimStruct({"szUserName": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 257), "dwSizeofEapInfo": SimTypeInt(signed=False, label="UInt32"), "pbEapInfo": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="RASEAPUSERIDENTITYA", pack=False, align=None), offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["pszPhonebook", "pszEntry", "dwFlags", "hwnd", "ppRasEapUserIdentity"]),
        #
        'RasFreeEapUserIdentityW': SimTypeFunction([SimTypePointer(SimStruct({"szUserName": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 257), "dwSizeofEapInfo": SimTypeInt(signed=False, label="UInt32"), "pbEapInfo": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="RASEAPUSERIDENTITYW", pack=False, align=None), offset=0)], SimTypeBottom(label="Void"), arg_names=["pRasEapUserIdentity"]),
        #
        'RasFreeEapUserIdentityA': SimTypeFunction([SimTypePointer(SimStruct({"szUserName": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 257), "dwSizeofEapInfo": SimTypeInt(signed=False, label="UInt32"), "pbEapInfo": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="RASEAPUSERIDENTITYA", pack=False, align=None), offset=0)], SimTypeBottom(label="Void"), arg_names=["pRasEapUserIdentity"]),
        #
        'RasDeleteSubEntryA': SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["pszPhonebook", "pszEntry", "dwSubentryId"]),
        #
        'RasDeleteSubEntryW': SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["pszPhonebook", "pszEntry", "dwSubEntryId"]),
        #
        'RasUpdateConnection': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimStruct({"version": SimTypeInt(signed=False, label="RASAPIVERSION"), "dwSize": SimTypeInt(signed=False, label="UInt32"), "dwFlags": SimTypeInt(signed=False, label="UInt32"), "dwIfIndex": SimTypeInt(signed=False, label="UInt32"), "localEndPoint": SimStruct({"dwType": SimTypeInt(signed=False, label="UInt32"), "Anonymous": SimUnion({"ipv4": SimStruct({"S_un": SimUnion({"S_un_b": SimStruct({"s_b1": SimTypeChar(label="Byte"), "s_b2": SimTypeChar(label="Byte"), "s_b3": SimTypeChar(label="Byte"), "s_b4": SimTypeChar(label="Byte")}, name="_S_un_b_e__Struct", pack=False, align=None), "S_un_w": SimStruct({"s_w1": SimTypeShort(signed=False, label="UInt16"), "s_w2": SimTypeShort(signed=False, label="UInt16")}, name="_S_un_w_e__Struct", pack=False, align=None), "S_addr": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")}, name="IN_ADDR", pack=False, align=None), "ipv6": SimStruct({"u": SimUnion({"Byte": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "Word": SimTypeFixedSizeArray(SimTypeShort(signed=False, label="UInt16"), 8)}, name="<anon>", label="None")}, name="IN6_ADDR", pack=False, align=None)}, name="<anon>", label="None")}, name="RASTUNNELENDPOINT", pack=False, align=None), "remoteEndPoint": SimStruct({"dwType": SimTypeInt(signed=False, label="UInt32"), "Anonymous": SimUnion({"ipv4": SimStruct({"S_un": SimUnion({"S_un_b": SimStruct({"s_b1": SimTypeChar(label="Byte"), "s_b2": SimTypeChar(label="Byte"), "s_b3": SimTypeChar(label="Byte"), "s_b4": SimTypeChar(label="Byte")}, name="_S_un_b_e__Struct", pack=False, align=None), "S_un_w": SimStruct({"s_w1": SimTypeShort(signed=False, label="UInt16"), "s_w2": SimTypeShort(signed=False, label="UInt16")}, name="_S_un_w_e__Struct", pack=False, align=None), "S_addr": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")}, name="IN_ADDR", pack=False, align=None), "ipv6": SimStruct({"u": SimUnion({"Byte": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "Word": SimTypeFixedSizeArray(SimTypeShort(signed=False, label="UInt16"), 8)}, name="<anon>", label="None")}, name="IN6_ADDR", pack=False, align=None)}, name="<anon>", label="None")}, name="RASTUNNELENDPOINT", pack=False, align=None)}, name="RASUPDATECONN", pack=False, align=None), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hrasconn", "lprasupdateconn"]),
        #
        'RasGetProjectionInfoEx': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimStruct({"version": SimTypeInt(signed=False, label="RASAPIVERSION"), "type": SimTypeInt(signed=False, label="RASPROJECTION_INFO_TYPE"), "Anonymous": SimUnion({"ppp": SimStruct({"dwIPv4NegotiationError": SimTypeInt(signed=False, label="UInt32"), "ipv4Address": SimStruct({"S_un": SimUnion({"S_un_b": SimStruct({"s_b1": SimTypeChar(label="Byte"), "s_b2": SimTypeChar(label="Byte"), "s_b3": SimTypeChar(label="Byte"), "s_b4": SimTypeChar(label="Byte")}, name="_S_un_b_e__Struct", pack=False, align=None), "S_un_w": SimStruct({"s_w1": SimTypeShort(signed=False, label="UInt16"), "s_w2": SimTypeShort(signed=False, label="UInt16")}, name="_S_un_w_e__Struct", pack=False, align=None), "S_addr": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")}, name="IN_ADDR", pack=False, align=None), "ipv4ServerAddress": SimStruct({"S_un": SimUnion({"S_un_b": SimStruct({"s_b1": SimTypeChar(label="Byte"), "s_b2": SimTypeChar(label="Byte"), "s_b3": SimTypeChar(label="Byte"), "s_b4": SimTypeChar(label="Byte")}, name="_S_un_b_e__Struct", pack=False, align=None), "S_un_w": SimStruct({"s_w1": SimTypeShort(signed=False, label="UInt16"), "s_w2": SimTypeShort(signed=False, label="UInt16")}, name="_S_un_w_e__Struct", pack=False, align=None), "S_addr": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")}, name="IN_ADDR", pack=False, align=None), "dwIPv4Options": SimTypeInt(signed=False, label="UInt32"), "dwIPv4ServerOptions": SimTypeInt(signed=False, label="UInt32"), "dwIPv6NegotiationError": SimTypeInt(signed=False, label="UInt32"), "bInterfaceIdentifier": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8), "bServerInterfaceIdentifier": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8), "fBundled": SimTypeInt(signed=True, label="Int32"), "fMultilink": SimTypeInt(signed=True, label="Int32"), "dwAuthenticationProtocol": SimTypeInt(signed=False, label="RASPPP_PROJECTION_INFO_SERVER_AUTH_PROTOCOL"), "dwAuthenticationData": SimTypeInt(signed=False, label="RASPPP_PROJECTION_INFO_SERVER_AUTH_DATA"), "dwServerAuthenticationProtocol": SimTypeInt(signed=False, label="RASPPP_PROJECTION_INFO_SERVER_AUTH_PROTOCOL"), "dwServerAuthenticationData": SimTypeInt(signed=False, label="RASPPP_PROJECTION_INFO_SERVER_AUTH_DATA"), "dwEapTypeId": SimTypeInt(signed=False, label="UInt32"), "dwServerEapTypeId": SimTypeInt(signed=False, label="UInt32"), "dwLcpOptions": SimTypeInt(signed=False, label="UInt32"), "dwLcpServerOptions": SimTypeInt(signed=False, label="UInt32"), "dwCcpError": SimTypeInt(signed=False, label="UInt32"), "dwCcpCompressionAlgorithm": SimTypeInt(signed=False, label="UInt32"), "dwCcpServerCompressionAlgorithm": SimTypeInt(signed=False, label="UInt32"), "dwCcpOptions": SimTypeInt(signed=False, label="UInt32"), "dwCcpServerOptions": SimTypeInt(signed=False, label="UInt32")}, name="RASPPP_PROJECTION_INFO", pack=False, align=None), "ikev2": SimStruct({"dwIPv4NegotiationError": SimTypeInt(signed=False, label="UInt32"), "ipv4Address": SimStruct({"S_un": SimUnion({"S_un_b": SimStruct({"s_b1": SimTypeChar(label="Byte"), "s_b2": SimTypeChar(label="Byte"), "s_b3": SimTypeChar(label="Byte"), "s_b4": SimTypeChar(label="Byte")}, name="_S_un_b_e__Struct", pack=False, align=None), "S_un_w": SimStruct({"s_w1": SimTypeShort(signed=False, label="UInt16"), "s_w2": SimTypeShort(signed=False, label="UInt16")}, name="_S_un_w_e__Struct", pack=False, align=None), "S_addr": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")}, name="IN_ADDR", pack=False, align=None), "ipv4ServerAddress": SimStruct({"S_un": SimUnion({"S_un_b": SimStruct({"s_b1": SimTypeChar(label="Byte"), "s_b2": SimTypeChar(label="Byte"), "s_b3": SimTypeChar(label="Byte"), "s_b4": SimTypeChar(label="Byte")}, name="_S_un_b_e__Struct", pack=False, align=None), "S_un_w": SimStruct({"s_w1": SimTypeShort(signed=False, label="UInt16"), "s_w2": SimTypeShort(signed=False, label="UInt16")}, name="_S_un_w_e__Struct", pack=False, align=None), "S_addr": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")}, name="IN_ADDR", pack=False, align=None), "dwIPv6NegotiationError": SimTypeInt(signed=False, label="UInt32"), "ipv6Address": SimStruct({"u": SimUnion({"Byte": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "Word": SimTypeFixedSizeArray(SimTypeShort(signed=False, label="UInt16"), 8)}, name="<anon>", label="None")}, name="IN6_ADDR", pack=False, align=None), "ipv6ServerAddress": SimStruct({"u": SimUnion({"Byte": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "Word": SimTypeFixedSizeArray(SimTypeShort(signed=False, label="UInt16"), 8)}, name="<anon>", label="None")}, name="IN6_ADDR", pack=False, align=None), "dwPrefixLength": SimTypeInt(signed=False, label="UInt32"), "dwAuthenticationProtocol": SimTypeInt(signed=False, label="UInt32"), "dwEapTypeId": SimTypeInt(signed=False, label="UInt32"), "dwFlags": SimTypeInt(signed=False, label="RASIKEV_PROJECTION_INFO_FLAGS"), "dwEncryptionMethod": SimTypeInt(signed=False, label="UInt32"), "numIPv4ServerAddresses": SimTypeInt(signed=False, label="UInt32"), "ipv4ServerAddresses": SimTypePointer(SimStruct({"S_un": SimUnion({"S_un_b": SimStruct({"s_b1": SimTypeChar(label="Byte"), "s_b2": SimTypeChar(label="Byte"), "s_b3": SimTypeChar(label="Byte"), "s_b4": SimTypeChar(label="Byte")}, name="_S_un_b_e__Struct", pack=False, align=None), "S_un_w": SimStruct({"s_w1": SimTypeShort(signed=False, label="UInt16"), "s_w2": SimTypeShort(signed=False, label="UInt16")}, name="_S_un_w_e__Struct", pack=False, align=None), "S_addr": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")}, name="IN_ADDR", pack=False, align=None), offset=0), "numIPv6ServerAddresses": SimTypeInt(signed=False, label="UInt32"), "ipv6ServerAddresses": SimTypePointer(SimStruct({"u": SimUnion({"Byte": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "Word": SimTypeFixedSizeArray(SimTypeShort(signed=False, label="UInt16"), 8)}, name="<anon>", label="None")}, name="IN6_ADDR", pack=False, align=None), offset=0)}, name="RASIKEV2_PROJECTION_INFO", pack=False, align=None)}, name="<anon>", label="None")}, name="RAS_PROJECTION_INFO", pack=False, align=None), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hrasconn", "pRasProjection", "lpdwSize"]),
    }

lib.set_prototypes(prototypes)



import os
import pickle
import sys
import logging
import subprocess
from functools import lru_cache
from typing import Optional, Sequence
from tempfile import NamedTemporaryFile

from unittest import skipIf, skipUnless, skip, SkipTest

l = logging.getLogger("angr.tests.common")

try:
    import tracer
except ImportError:
    tracer = None

bin_location = os.path.join(os.path.dirname(os.path.realpath(__file__)), "..", "..", "binaries")
bin_priv_location = os.path.join(os.path.dirname(os.path.realpath(__file__)), "..", "..", "binaries-private")

if not os.path.isdir(bin_location):
    raise Exception(
        "Can't find the angr/binaries repo for holding testcases. "
        "It should be cloned into the same folder as the rest of your angr modules."
    )


def broken(func):
    return skip(reason="Broken test method")(func)


def requires_binaries_private(func):
    return skipIf(
        not os.path.exists(
            os.path.join(os.path.dirname(os.path.dirname(os.path.dirname(__file__))), "binaries-private"),
        ),
        "Skip this test since we do not have the binaries-private repo cloned on Travis CI.",
    )(func)


def slow_test(func):
    func.speed = "slow"
    slow_test_env = os.environ["SKIP_SLOW_TESTS"].lower() if "SKIP_SLOW_TESTS" in os.environ else ""
    return skipIf(slow_test_env == "true" or slow_test_env == "1", "Skipping slow test")(func)


def skip_if_not_linux(func):
    return skipUnless(sys.platform.startswith("linux"), "Skipping Linux Test Cases")(func)


TRACE_VERSION = 1


def do_trace(proj, test_name, input_data, **kwargs):
    """
    trace, magic, crash_mode, crash_addr = load_cached_trace(proj, "test_blurble")
    """
    fname = os.path.join(
        bin_location,
        "tests_data",
        "runner_traces",
        f"{test_name}_{os.path.basename(proj.filename)}_{proj.arch.name}.p",
    )

    if os.path.isfile(fname):
        try:
            with open(fname, "rb") as f:
                r = pickle.load(f)
                if type(r) is tuple and len(r) == 2 and r[1] == TRACE_VERSION:
                    return r[0]
        except (pickle.UnpicklingError, UnicodeDecodeError):
            print("Can't unpickle trace - rerunning")

    if tracer is None:
        raise SkipTest("Tracer is not installed and cached data is not present")

    runner = tracer.QEMURunner(project=proj, input=input_data, **kwargs)
    r = (runner.trace, runner.magic, runner.crash_mode, runner.crash_addr)
    with open(fname, "wb") as f:
        pickle.dump((r, TRACE_VERSION), f, -1)
    return r


@skipUnless(tracer, "tracer is not installed")
def load_cgc_pov(pov_file: str) -> "tracer.TracerPoV":
    return tracer.TracerPoV(pov_file)


def compile_c(c_code: str, cflags: Optional[Sequence[str]], silent: bool = False) -> NamedTemporaryFile:
    # pylint:disable=consider-using-with
    """
    Compile `c_code` and return the file containing the compiled output
    """
    dst = None
    try:
        dst = NamedTemporaryFile(delete=False)
        dst.close()
        src = NamedTemporaryFile(mode="x", delete=False, suffix=".c")
        src.write(c_code)
        src.close()

        call_args = ["cc"] + (cflags or []) + ["-o", dst.name, src.name]
        l.debug("Compiling with: %s", " ".join(call_args))
        l.debug("Source:\n%s", c_code)
        out = subprocess.DEVNULL if silent else None
        subprocess.check_call(call_args, stderr=out, stdout=out)
        return dst
    except:
        if dst and os.path.exists(dst.name):
            os.remove(dst.name)
        raise
    finally:
        if src and os.path.exists(src.name):
            os.remove(src.name)


@lru_cache
def has_32_bit_compiler_support() -> bool:
    """
    Check if we are able to compile a 32-bit binary
    """
    try:
        binary = compile_c("#include <stdlib.h>\nint main() { return 0; }", ["-m32"], True)
        os.remove(binary.name)
        return True
    except subprocess.CalledProcessError:
        return False



#!/usr/bin/env python3
__package__ = __package__ or "tests.analyses.decompiler"  # pylint:disable=redefined-builtin

import os.path
import unittest

import angr
import ailment
from angr.analyses.decompiler.optimization_passes.base_ptr_save_simplifier import (
    BasePointerSaveSimplifier,
)

from ...common import bin_location


test_location = os.path.join(bin_location, "tests")


def _get_block(clinic, addr):
    for block in clinic.graph.nodes():
        if block.addr == addr:
            return block
    return None


def check_bp_save_fauxware(arch):
    p = angr.Project(os.path.join(test_location, arch, "fauxware"), auto_load_libs=False)
    cfg = p.analyses.CFGFast(normalize=True)
    main = p.kb.functions["main"]
    optimization_passes = [BasePointerSaveSimplifier]
    dra = p.analyses.Decompiler(main, cfg=cfg, optimization_passes=optimization_passes)
    first_block_stmts = dra.codegen._sequence.nodes[0].nodes[0].statements
    for stmt in first_block_stmts:
        if isinstance(stmt, ailment.Stmt.Store):
            assert not (isinstance(stmt.data, ailment.Expr.Register) and stmt.data.reg_offset == p.arch.bp_offset) or (
                isinstance(stmt.data, ailment.Expr.StackBaseOffset) and stmt.data.offset == 0
            )


# pylint: disable=missing-class-docstring
# pylint: disable=no-self-use
class TestBaseptrSaveSimplifier(unittest.TestCase):
    def test_baseptr_save_simplifier_amd64(self):
        # decompile all:main and make sure the first and the last blocks do not save or restore to rbp
        bin_path = os.path.join(test_location, "x86_64", "all")
        proj = angr.Project(bin_path, auto_load_libs=False, load_debug_info=True)

        cfg = proj.analyses.CFG(data_references=True, normalize=True)

        optimization_passes = [BasePointerSaveSimplifier]
        main_func = cfg.functions["main"]
        dec = proj.analyses.Decompiler(main_func, cfg=cfg, optimization_passes=optimization_passes)

        entry_block = _get_block(dec.clinic, main_func.addr)
        endpoint_block = _get_block(dec.clinic, next(iter(main_func.endpoints)).addr)

        assert entry_block is not None
        assert endpoint_block is not None

        for stmt in entry_block.statements:
            if isinstance(stmt, ailment.Stmt.Store) and isinstance(stmt.data, ailment.Expr.StackBaseOffset):
                assert False, "Found a base-pointer saving statement in the first block."

        for stmt in endpoint_block.statements:
            if (
                isinstance(stmt, ailment.Stmt.Assignment)
                and isinstance(stmt.dst, ailment.Expr.Register)
                and stmt.dst.reg_offset == proj.arch.bp_offset
            ):
                assert False, "Found a base-pointer restoring statement in the last block."

    def test_bp_save_amd64_fauxware(self):
        check_bp_save_fauxware("x86_64")

    def test_bp_save_armel_fauxware(self):
        check_bp_save_fauxware("armel")


if __name__ == "__main__":
    unittest.main()



import angr


class sigprocmask(angr.SimProcedure):
    # pylint:disable=arguments-differ

    def run(self, how, set_, oldset, sigsetsize):
        self.state.memory.store(oldset, self.state.posix.sigmask(sigsetsize=sigsetsize), condition=oldset != 0)
        self.state.posix.sigprocmask(how, self.state.memory.load(set_, sigsetsize), sigsetsize, valid_ptr=set_ != 0)

        # TODO: EFAULT
        return self.state.solver.If(
            self.state.solver.And(
                how != self.state.posix.SIG_BLOCK,
                how != self.state.posix.SIG_UNBLOCK,
                how != self.state.posix.SIG_SETMASK,
            ),
            self.state.solver.BVV(self.state.posix.EINVAL, self.arch.sizeof["int"]),
            0,
        )



# pylint:disable=no-self-use,arguments-renamed
from collections import OrderedDict

import ailment

from ..structuring.structurer_nodes import SwitchCaseNode
from ..sequence_walker import SequenceWalker


class SwitchExpressionSimplifier(SequenceWalker):
    """
    Identifies switch expressions that adds or minuses a constant, removes the constant from the switch expression, and
    adjust all case expressions accordingly.
    """

    def __init__(self, node):
        handlers = {
            SwitchCaseNode: self._handle_SwitchCase,
        }
        super().__init__(handlers)

        self.walk(node)

    def _handle_SwitchCase(self, node: SwitchCaseNode, **kwargs):
        changed = False

        switch_expr = node.switch_expr
        convert = None

        # pylint: disable=import-outside-toplevel
        from ..peephole_optimizations.remove_noop_conversions import RemoveNoopConversions

        while isinstance(switch_expr, ailment.Expr.Convert):
            optimized = RemoveNoopConversions(None, None).optimize(switch_expr)
            if optimized is not None:
                switch_expr = optimized
                continue

            convert = switch_expr
            switch_expr = switch_expr.operand

        if isinstance(switch_expr, ailment.Expr.BinaryOp):
            if switch_expr.op in {"Add", "Sub"} and isinstance(switch_expr.operands[1], ailment.Expr.Const):
                v = switch_expr.operands[1].value
                if switch_expr.op == "Add":
                    v = -v

                new_switch_expr = switch_expr.operands[0]
                if convert is not None:
                    # unpack if necessary
                    if isinstance(new_switch_expr, ailment.Expr.Convert):
                        if new_switch_expr.to_bits == convert.from_bits:
                            new_switch_expr = new_switch_expr.operand
                        else:
                            new_switch_expr = ailment.Expr.Convert(
                                None,
                                new_switch_expr.from_bits,
                                convert.to_bits,
                                convert.is_signed,
                                new_switch_expr.operand,
                                **convert.tags,
                            )
                    else:
                        new_switch_expr = ailment.Expr.Convert(
                            None, convert.from_bits, convert.to_bits, convert.is_signed, new_switch_expr, **convert.tags
                        )

                new_cases = OrderedDict()
                for case_idx, case_node in node.cases.items():
                    if isinstance(case_idx, int):
                        new_cases[case_idx + v] = case_node
                    elif isinstance(case_idx, tuple):
                        new_cases[tuple(idx_ + v for idx_ in case_idx)] = case_node
                    else:
                        raise TypeError(f"Unsupported case_idx type {type(case_idx)}")
                new_node = SwitchCaseNode(new_switch_expr, new_cases, default_node=node.default_node, addr=node.addr)
                node = new_node
                changed = True

        r = super()._handle_SwitchCase(node, **kwargs)
        if changed and r is None:
            return node
        return r



import inspect
import copy
import itertools
import logging
from typing import TYPE_CHECKING, Union, Tuple

import claripy
from cle import SymbolType
from archinfo.arch_soot import SootAddressDescriptor

if TYPE_CHECKING:
    import angr
    import archinfo
    from angr.sim_state import SimState

l = logging.getLogger(name=__name__)
symbolic_count = itertools.count()


class SimProcedure:
    """
    A SimProcedure is a wonderful object which describes a procedure to run on a state.

    You may subclass SimProcedure and override ``run()``, replacing it with mutating ``self.state`` however you like,
    and then either returning a value or jumping away somehow.

    A detailed discussion of programming SimProcedures may be found at https://docs.angr.io/extending-angr/simprocedures

    :param arch:            The architecture to use for this procedure

    The following parameters are optional:

    :param symbolic_return: Whether the procedure's return value should be stubbed into a
                            single symbolic variable constratined to the real return value
    :param returns:         Whether the procedure should return to its caller afterwards
    :param is_syscall:      Whether this procedure is a syscall
    :param num_args:        The number of arguments this procedure should extract
    :param display_name:    The name to use when displaying this procedure
    :param library_name:    The name of the library from which the function we're emulating comes
    :param cc:              The SimCC to use for this procedure
    :param sim_kwargs:      Additional keyword arguments to be passed to run()
    :param is_function:     Whether this procedure emulates a function

    The following class variables should be set if necessary when implementing a new SimProcedure:

    :cvar NO_RET:           Set this to true if control flow will never return from this function
    :cvar DYNAMIC_RET:      Set this to true if whether the control flow returns from this function or not depends on
                            the context (e.g., libc's error() call). Must implement dynamic_returns() method.
    :cvar ADDS_EXITS:       Set this to true if you do any control flow other than returning
    :cvar IS_FUNCTION:      Does this procedure simulate a function? True by default
    :cvar ARGS_MISMATCH:    Does this procedure have a different list of arguments than what is provided in the
                            function specification? This may happen when we manually extract arguments in the run()
                            method of a SimProcedure. False by default.
    :cvar local_vars:       If you use ``self.call()``, set this to a list of all the local variable
                            names in your class. They will be restored on return.

    The following instance variables are available when working with simprocedures from the inside or the outside:

    :ivar project:          The associated angr project
    :ivar arch:             The associated architecture
    :ivar addr:             The linear address at which the procedure is executing
    :ivar cc:               The calling convention in use for engaging with the ABI
    :ivar canonical:        The canonical version of this SimProcedure. Procedures are deepcopied for many reasons,
                            including to be able to store state related to a specific run and to be able to hook
                            continuations.
    :ivar kwargs:           Any extra keyword arguments used to construct the procedure; will be passed to ``run``
    :ivar display_name:     See the eponymous parameter
    :ivar library_name:     See the eponymous parameter
    :ivar abi:              If this is a syscall simprocedure, which ABI are we using to map the syscall numbers?
    :ivar symbolic_return:  See the eponymous parameter
    :ivar syscall_number:   If this procedure is a syscall, the number will be populated here.
    :ivar returns:          See eponymous parameter and NO_RET cvar
    :ivar is_syscall:       See eponymous parameter
    :ivar is_function:      See eponymous parameter and cvar
    :ivar is_stub:          See eponymous parameter
    :ivar is_continuation:  Whether this procedure is the original or a continuation resulting from ``self.call()``
    :ivar continuations:    A mapping from name to each known continuation
    :ivar run_func:         The name of the function implementing the procedure. "run" by default, but different in
                            continuations.
    :ivar num_args:         The number of arguments to the procedure. If not provided in the parameter, extracted from
                            the definition of ``self.run``

    The following instance variables are only used in a copy of the procedure that is actually executing on a state:

    :ivar state:            The SimState we should be mutating to perform the procedure
    :ivar successors:       The SimSuccessors associated with the current step
    :ivar arguments:        The function arguments, deserialized from the state
    :ivar arg_session:      The ArgSession that was used to parse arguments out of the state, in case you need it for
                            varargs
    :ivar use_state_arguments:
                            Whether we're using arguments extracted from the state or manually provided
    :ivar ret_to:           The current return address
    :ivar ret_expr:         The computed return value
    :ivar call_ret_expr:    The return value from having used ``self.call()``
    :ivar inhibit_autoret:  Whether we should avoid automatically adding an exit for returning once the run function
                            ends
    :ivar arg_session:      The ArgSession object that was used to extract the runtime argument values. Useful for if
                            you want to extract variadic args.

    """

    state: "SimState"

    def __init__(
        self,
        project=None,
        cc=None,
        prototype=None,
        symbolic_return=None,
        returns=None,
        is_syscall=False,
        is_stub=False,
        num_args=None,
        display_name=None,
        library_name=None,
        is_function=None,
        **kwargs,
    ):
        # WE'LL FIGURE IT OUT
        self.project: angr.Project = project
        self.arch: archinfo.arch.Arch = project.arch if project is not None else None
        self.addr = None
        self.cc: angr.SimCC = cc
        if type(prototype) is str:
            prototype = parse_signature(prototype)
        self.prototype: angr.sim_type.SimTypeFunction = prototype
        self.canonical = self

        self.kwargs = kwargs
        self.display_name = type(self).__name__ if display_name is None else display_name
        self.library_name = library_name
        self.syscall_number = None
        self.abi = None
        self.symbolic_return = symbolic_return

        # set some properties about the type of procedure this is
        self.returns = returns if returns is not None else not self.NO_RET
        self.is_syscall = is_syscall
        self.is_function = is_function if is_function is not None else self.IS_FUNCTION
        self.is_stub = is_stub
        self.is_continuation = False
        self.continuations = {}
        self.run_func = "run"

        # Get the concrete number of arguments that should be passed to this procedure
        if num_args is None:
            run_spec = inspect.getfullargspec(self.run)
            self.num_args = len(run_spec.args) - (len(run_spec.defaults) if run_spec.defaults is not None else 0) - 1
        else:
            self.num_args = num_args

        if self.prototype is None:
            charp = SimTypePointer(SimTypeChar())
            self.prototype = SimTypeFunction([charp] * self.num_args, charp)
            self.guessed_prototype = True
        else:
            self.guessed_prototype = False

        # runtime values
        self.state = None
        self.successors = None
        self.arguments = None
        self.use_state_arguments = True
        self.ret_to = None
        self.ret_expr = None
        self.call_ret_expr = None
        self.inhibit_autoret = None
        self.arg_session: Union[None, ArgSession, int] = None

    def __repr__(self):
        return "<SimProcedure %s%s%s%s%s>" % self._describe_me()

    def _describe_me(self):
        """
        return a 5-tuple of strings sufficient for formatting with ``%s%s%s%s%s`` to verbosely describe the procedure
        """
        return (
            self.display_name,
            " (cont: %s)" % self.run_func if self.is_continuation else "",
            " (syscall)" if self.is_syscall else "",
            " (inline)" if not self.use_state_arguments else "",
            " (stub)" if self.is_stub else "",
        )

    def execute(self, state, successors=None, arguments=None, ret_to=None):
        """
        Call this method with a SimState and a SimSuccessors to execute the procedure.

        Alternately, successors may be none if this is an inline call. In that case, you should
        provide arguments to the function.
        """
        # fill out all the fun stuff we don't want to frontload
        if self.addr is None and not state.regs._ip.symbolic:
            self.addr = state.addr
        if self.arch is None:
            self.arch = state.arch
        if self.project is None:
            self.project = state.project
        if self.cc is None:
            if self.arch.name in DEFAULT_CC:
                self.cc = default_cc(
                    self.arch.name,
                    platform=self.project.simos.name
                    if self.project is not None and self.project.simos is not None
                    else None,
                )(self.arch)
            else:
                raise SimProcedureError(
                    "There is no default calling convention for architecture %s."
                    " You must specify a calling convention." % self.arch.name
                )
        if self.prototype._arch is None:
            self.prototype = self.prototype.with_arch(self.arch)

        inst = copy.copy(self)
        inst.state = state
        inst.successors = successors
        inst.ret_to = ret_to
        inst.inhibit_autoret = False

        # check to see if this is a syscall and if we should override its return value
        if inst.is_syscall:
            state.history.recent_syscall_count = 1

        state._inspect(
            "simprocedure",
            BP_BEFORE,
            simprocedure_name=inst.display_name,
            simprocedure_addr=self.addr,
            simprocedure=inst,
            simprocedure_result=NO_OVERRIDE,
        )

        r = state._inspect_getattr("simprocedure_result", NO_OVERRIDE)
        if r is NO_OVERRIDE:
            # get the arguments

            # If the simprocedure is related to a Java function call the appropriate setup_args methos
            # TODO: should we move this?
            if self.is_java:
                sim_args = self._setup_args(inst, state, arguments)  # pylint:disable=assignment-from-no-return
                self.use_state_arguments = False

            # handle if this is a continuation from a return
            elif inst.is_continuation:
                if state.callstack.top.procedure_data is None:
                    raise SimProcedureError("Tried to return to a SimProcedure in an inapplicable stack frame!")

                saved_sp, sim_args, saved_local_vars, saved_lr, ideal_addr = state.callstack.top.procedure_data
                if ideal_addr != inst.addr:
                    raise SimShadowStackError("I can't emulate this consequence of stack smashing")
                state.regs.sp = saved_sp
                if saved_lr is not None:
                    state.regs.lr = saved_lr
                inst.arguments = sim_args
                inst.use_state_arguments = True
                inst.call_ret_expr = state.registers.load(
                    state.arch.ret_offset, state.arch.bytes, endness=state.arch.register_endness
                )
                for name, val in saved_local_vars:
                    setattr(inst, name, val)
            else:
                if arguments is None:
                    inst.use_state_arguments = True
                    inst.arg_session = inst.cc.arg_session(inst.prototype.returnty)
                    sim_args = [
                        inst.cc.next_arg(inst.arg_session, ty).get_value(inst.state) for ty in inst.prototype.args
                    ]
                    inst.arguments = sim_args
                else:
                    inst.use_state_arguments = False
                    sim_args = arguments[: inst.num_args]
                    inst.arguments = arguments
                    inst.arg_session = 0

            # run it
            l.debug("Executing %s%s%s%s%s with %s, %s", *(inst._describe_me() + (sim_args, inst.kwargs)))
            r = getattr(inst, inst.run_func)(*sim_args, **inst.kwargs)

        state._inspect(
            "simprocedure",
            BP_AFTER,
            simprocedure_name=inst.display_name,
            simprocedure_addr=self.addr,
            simprocedure=inst,
            simprocedure_result=r,
        )
        r = state._inspect_getattr("simprocedure_result", r)

        if inst.returns and inst.is_function and not inst.inhibit_autoret:
            inst.ret(r)

        return inst

    def make_continuation(self, name):
        # make a copy of the canon copy, customize it for the specific continuation, then hook it
        if name not in self.canonical.continuations:
            cont = copy.copy(self.canonical)
            target_name = f"{self.display_name}.{name}"
            should_be_none = self.project.loader.extern_object.get_symbol(target_name)
            if should_be_none is None:
                cont.addr = self.project.loader.extern_object.make_extern(
                    target_name, sym_type=SymbolType.TYPE_OTHER
                ).rebased_addr
            else:
                l.error("Trying to make continuation %s but it already exists. This is bad.", target_name)
                cont.addr = self.project.loader.extern_object.allocate()
            cont.is_continuation = True
            cont.run_func = name
            self.canonical.continuations[name] = cont
            self.project.hook(cont.addr, cont)
        return self.canonical.continuations[name].addr

    #
    # Implement these in a subclass of SimProcedure!
    #

    NO_RET = False
    DYNAMIC_RET = False
    ADDS_EXITS = False
    IS_FUNCTION = True
    ARGS_MISMATCH = False
    ALT_NAMES = None  # alternative names
    local_vars: Tuple[str, ...] = ()

    def run(self, *args, **kwargs):  # pylint: disable=unused-argument
        """
        Implement the actual procedure here!
        """
        raise SimProcedureError("%s does not implement a run() method" % self.__class__.__name__)

    def static_exits(self, blocks, **kwargs):  # pylint: disable=unused-argument
        """
        Get new exits by performing static analysis and heuristics. This is a fast and best-effort approach to get new
        exits for scenarios where states are not available (e.g. when building a fast CFG).

        :param list blocks: Blocks that are executed before reaching this SimProcedure.
        :return: A list of dicts. Each dict should contain the following entries: 'address', 'jumpkind', and 'namehint'.
        :rtype: list
        """

        if self.ADDS_EXITS:
            raise SimProcedureError("static_exits() is not implemented for %s" % self)

        # This SimProcedure does not add any new exit
        return []

    def dynamic_returns(self, blocks, **kwargs) -> bool:  # pylint:disable=unused-argument
        """
        Determines if a call to this function returns or not by performing static analysis and heuristics.

        :param blocks:  Blocks that are executed before reaching this SimProcedure.
        :return:        True if the call returns, False otherwise.
        """

        if self.DYNAMIC_RET:
            raise SimProcedureError(f"dynamic_returns() is not implemented for {self}")

        return True

    #
    # misc properties
    #

    @property
    def should_add_successors(self):
        return self.successors is not None

    #
    # Working with calling conventions
    #

    def _setup_args(self, inst, state, args):  # pylint:disable=unused-argument,no-self-use
        raise SimProcedureError("the java-specific _setup_args() method was invoked on a non-Java SimProcedure.")

    def _compute_ret_addr(self, expr):  # pylint:disable=unused-argument,no-self-use
        raise SimProcedureError("the java-specific _compute_ret_addr() method was invoked on a non-Java SimProcedure.")

    def set_args(self, args):
        arg_session = self.cc.arg_session(self.prototype.returnty)
        for arg, ty in zip(args, self.prototype.args):
            self.cc.next_arg(arg_session, ty).set_value(self.state, arg)

    def va_arg(self, ty, index=None):
        if not self.use_state_arguments:
            if index is not None:
                return self.arguments[self.num_args + index]

            result = self.arguments[self.num_args + self.arg_session]
            self.arg_session += 1
            return result

        if index is not None:
            raise Exception("you think you're so fucking smart? you implement this logic then")

        if type(ty) is str:
            ty = parse_type(ty, arch=self.arch)
        return self.cc.next_arg(self.arg_session, ty).get_value(self.state)

    #
    # Control Flow
    #

    def inline_call(self, procedure, *arguments, **kwargs):
        """
        Call another SimProcedure in-line to retrieve its return value.
        Returns an instance of the procedure with the ret_expr property set.

        :param procedure:       The class of the procedure to execute
        :param arguments:       Any additional positional args will be used as arguments to the
                                procedure call
        :param sim_kwargs:      Any additional keyword args will be passed as sim_kwargs to the
                                procedure construtor
        """
        e_args = [self.state.solver.BVV(a, self.state.arch.bits) if isinstance(a, int) else a for a in arguments]
        p = procedure(project=self.project, **kwargs)
        return p.execute(self.state, None, arguments=e_args)

    def fix_prototype_returnty(self, ret_size):
        if ret_size not in [8, 16, 32, 64]:
            raise NotImplementedError(f"return type of size {ret_size} is not handled!")
        returnty = SimTypeNum(ret_size, signed=False, label=f"u{ret_size}")
        self.prototype.returnty = returnty

    def ret(self, expr=None):
        """
        Add an exit representing a return from this function.
        If this is not an inline call, grab a return address from the state and jump to it.
        If this is not an inline call, set a return expression with the calling convention.
        """
        self.inhibit_autoret = True

        if expr is not None:
            if o.SIMPLIFY_RETS in self.state.options:
                l.debug("... simplifying")
                l.debug("... before: %s", expr)
                expr = self.state.solver.simplify(expr)
                l.debug("... after: %s", expr)

            if self.symbolic_return:
                size = len(expr)
                new_expr = self.state.solver.Unconstrained(
                    "symbolic_return_" + self.display_name, size, key=("symbolic_return", self.display_name)
                )  # pylint:disable=maybe-no-member
                self.state.add_constraints(new_expr == expr)
                expr = new_expr

            self.ret_expr = expr

        ret_addr = None
        # TODO: I had to put this check here because I don't understand why self.use_state_arguments gets reset to true
        # when calling the function ret. at the calling point the attribute is set to False
        if isinstance(self.addr, SootAddressDescriptor):
            ret_addr = self._compute_ret_addr(expr)  # pylint:disable=assignment-from-no-return
        elif self.use_state_arguments:
            # in case we guess the prototype wrong, use the return value's size as a hint to fix it
            if (
                self.guessed_prototype
                and isinstance(expr, claripy.ast.bv.BV)
                and self.prototype.returnty.size != len(expr)
            ):
                self.fix_prototype_returnty(len(expr))

            ret_addr = self.cc.teardown_callsite(self.state, return_val=expr, prototype=self.prototype)

        if not self.should_add_successors:
            l.debug("Returning without setting exits due to 'internal' call.")
            return

        if self.ret_to is not None:
            ret_addr = self.ret_to

        if ret_addr is None:
            raise SimProcedureError("No source for return address in ret() call!")

        self._prepare_ret_state()

        self._exit_action(self.state, ret_addr)
        self.successors.add_successor(self.state, ret_addr, self.state.solver.true, "Ijk_Ret")

    def call(self, addr, args, continue_at, cc=None, prototype=None, jumpkind="Ijk_Call"):
        """
        Add an exit representing calling another function via pointer.

        :param addr:        The address of the function to call
        :param args:        The list of arguments to call the function with
        :param continue_at: Later, when the called function returns, execution of the current
                            procedure will continue in the named method.
        :param cc:          Optional: use this calling convention for calling the new function.
                            Default is to use the current convention.
        :param prototype:     Optional: The prototype to use for the call. Will default to all-ints.
        """
        self.inhibit_autoret = True

        if cc is None:
            cc = self.cc
        prototype = cc.guess_prototype(args, prototype)

        call_state = self.state.copy()
        ret_addr = self.make_continuation(continue_at)
        saved_local_vars = list(zip(self.local_vars, map(lambda name: getattr(self, name), self.local_vars)))
        simcallstack_entry = (
            self.state.regs.sp if hasattr(self.state.regs, "sp") else None,
            self.arguments,
            saved_local_vars,
            self.state.regs.lr if self.state.arch.lr_offset is not None else None,
            ret_addr,
        )
        cc.setup_callsite(call_state, ret_addr, args, prototype)
        call_state.callstack.top.procedure_data = simcallstack_entry

        # TODO: Move this to setup_callsite?
        if isinstance(call_state.addr, SootAddressDescriptor):
            pass
        elif call_state.libc.ppc64_abiv == "ppc64_1":
            call_state.regs.r2 = self.state.mem[addr + 8 :].long.resolved
            addr = call_state.mem[addr:].long.resolved
        elif call_state.arch.name in ("MIPS32", "MIPS64"):
            call_state.regs.t9 = addr

        self._exit_action(call_state, addr)
        self.successors.add_successor(call_state, addr, call_state.solver.true, jumpkind)

        if o.DO_RET_EMULATION in self.state.options:
            # we need to set up the call because the continuation will try to tear it down
            ret_state = self.state.copy()
            cc.setup_callsite(ret_state, ret_addr, args, prototype)
            ret_state.callstack.top.procedure_data = simcallstack_entry
            guard = ret_state.solver.true if o.TRUE_RET_EMULATION_GUARD in ret_state.options else ret_state.solver.false
            self.successors.add_successor(ret_state, ret_addr, guard, "Ijk_FakeRet")

    def jump(self, addr, jumpkind="Ijk_Boring"):
        """
        Add an exit representing jumping to an address.
        """
        self.inhibit_autoret = True
        self._exit_action(self.state, addr)
        self.successors.add_successor(self.state, addr, self.state.solver.true, jumpkind)

    def exit(self, exit_code):
        """
        Add an exit representing terminating the program.
        """
        self.inhibit_autoret = True
        self.state.options.discard(o.AST_DEPS)
        self.state.options.discard(o.AUTO_REFS)

        if isinstance(exit_code, int):
            exit_code = self.state.solver.BVV(exit_code, self.state.arch.bits)
        self.state.history.add_event("terminate", exit_code=exit_code)
        self.successors.add_successor(self.state, self.state.regs.ip, self.state.solver.true, "Ijk_Exit")

    @staticmethod
    def _exit_action(state, addr):
        if o.TRACK_JMP_ACTIONS in state.options:
            state.history.add_action(SimActionExit(state, addr))

    #
    # misc
    #

    def ty_ptr(self, ty):
        return SimTypePointer(self.arch, ty)

    @property
    def is_java(self):
        return False

    def _prepare_ret_state(self):
        pass

    @property
    def argument_types(self):  # pylint: disable=no-self-use
        return None

    @argument_types.setter
    def argument_types(self, v):  # pylint: disable=unused-argument,no-self-use
        l.critical("SimProcedure.argument_types is deprecated. specify the function signature in the prototype param")

    @property
    def return_type(self):  # pylint: disable=no-self-use
        return None

    @return_type.setter
    def return_type(self, v):  # pylint: disable=unused-argument,no-self-use
        l.critical("SimProcedure.return_type is deprecated. specify the function signature in the prototype param")


from . import sim_options as o
from angr.errors import SimProcedureError, SimShadowStackError
from angr.state_plugins.sim_action import SimActionExit
from angr.calling_conventions import (
    DEFAULT_CC,
    default_cc,
    SimTypeFunction,
    SimTypePointer,
    SimTypeChar,
    ArgSession,
    SimTypeNum,
)
from .state_plugins import BP_AFTER, BP_BEFORE, NO_OVERRIDE
from .sim_type import parse_signature, parse_type



#!/usr/bin/env python3
# pylint: disable=missing-class-docstring,no-self-use,line-too-long

import binascii
import unittest

import pyvex
import archinfo
import angr


class TestLifter(unittest.TestCase):
    def test_strict_block_ends_cbz(self):
        # ldr     r3, [sp, #4]
        # cbz     r3, #0x8149
        # mov.w   r2, #0x10000000
        # ldr     r3, [pc, #0x38]
        # str     r2, [r3]
        # add     sp, #8
        # pop     {r4, r5, r6, pc}

        p = angr.load_shellcode(b"\x01\x9b\x1b\xb1O\xf0\x80R\x0eK\x1a`\x02\xb0p\xbd", "arm")
        assert p.factory.block(1, strict_block_end=False).instructions == 7
        assert p.factory.block(1, strict_block_end=True).instructions == 2
        p.factory.default_engine.default_strict_block_end = False
        assert p.factory.block(1).instructions == 7
        p.factory.default_engine.default_strict_block_end = True
        assert p.factory.block(1).instructions == 2

    def test_strict_block_ends_with_size_x86(self):
        # cmovnz  cx, dx
        # pop     eax
        # jecxz   short loc_4010D7  ; the first block should end here
        # sub     edi, 2
        # loop    loc_4010c9
        # nop
        # nop

        b = b"\x66\x0f\x45\xca\x58\xe3\x07\x83\xef\x02\xe2\xf4\x90\x90"
        p = angr.load_shellcode(b, "x86", load_address=0x4010C9)

        # jecxz
        assert p.factory.block(0x4010C9, size=len(b), strict_block_end=False).instructions == 7
        assert p.factory.block(0x4010C9, strict_block_end=True).instructions == 3
        assert p.factory.block(0x4010C9, size=len(b), strict_block_end=True).instructions == 3

        # loop
        assert p.factory.block(0x4010D0, strict_block_end=False).instructions == 4
        assert p.factory.block(0x4010D0, strict_block_end=True).instructions == 2
        assert p.factory.block(0x4010D0, size=7, strict_block_end=True).instructions == 2

    def test_strict_block_ends_with_size_amd64(self):
        # cmovnz  cx, dx
        # pop     rax
        # jrcxz   short loc_4010D7  ; the first block should end here
        # sub     edi, 2
        # loop    loc_4010c9
        # nop
        # nop

        b = b"\x66\x0f\x45\xca\x58\xe3\x07\x83\xef\x02\xe2\xf4\x90\x90"
        p = angr.load_shellcode(b, "amd64", load_address=0x4010C9)

        # jrcxz
        assert p.factory.block(0x4010C9, size=len(b), strict_block_end=False).instructions == 7
        assert p.factory.block(0x4010C9, strict_block_end=True).instructions == 3
        assert p.factory.block(0x4010C9, size=len(b), strict_block_end=True).instructions == 3

        # loop
        assert p.factory.block(0x4010D0, strict_block_end=False).instructions == 4
        assert p.factory.block(0x4010D0, strict_block_end=True).instructions == 2
        assert p.factory.block(0x4010D0, size=7, strict_block_end=True).instructions == 2

    def test_no_cross_insn_boundary_opt_amd64(self):
        # 0x4020f8:       sub     rsp, 8
        # 0x4020fc:       mov     rax, qword ptr [rip + 0x221ef5]
        # 0x402103:       test    rax, rax
        # 0x402106:       je      0x40210d

        b = binascii.unhexlify("4883ec08488b05f51e22004885c07405")
        p = angr.load_shellcode(b, "amd64", load_address=0x4020F8)

        # No optimization
        block = p.factory.block(0x4020F8, size=len(b), opt_level=0)
        assert len(block.vex.statements) == 32
        # Full level-1 optimization
        block = p.factory.block(0x4020F8, size=len(b), opt_level=1, cross_insn_opt=True)
        assert len(block.vex.statements) == 20
        # Level-1 optimization within each instruction
        block = p.factory.block(0x4020F8, size=len(b), opt_level=1, cross_insn_opt=False)
        stmts = block.vex.statements
        assert len(stmts) == 22
        # 09 | ------ IMark(0x402103, 3, 0) ------
        assert isinstance(stmts[9], pyvex.IRStmt.IMark)
        assert stmts[9].addr == 0x402103
        # 10 | t6 = GET:I64(rax)
        assert isinstance(stmts[10], pyvex.IRStmt.WrTmp)
        assert isinstance(stmts[10].data, pyvex.IRExpr.Get)
        assert stmts[10].data.offset == archinfo.arch_from_id("amd64").registers["rax"][0]
        # 11 | PUT(cc_op) = 0x0000000000000014
        assert isinstance(stmts[11], pyvex.IRStmt.Put)
        assert stmts[11].offset == archinfo.arch_from_id("amd64").registers["cc_op"][0]
        assert isinstance(stmts[11].data, pyvex.IRExpr.Const)
        assert stmts[11].data.con.value == 0x14
        # 12 | PUT(cc_dep1) = t6
        assert isinstance(stmts[12], pyvex.IRStmt.Put)
        assert stmts[12].offset == archinfo.arch_from_id("amd64").registers["cc_dep1"][0]
        # 13 | PUT(cc_dep2) = 0x0000000000000000
        assert isinstance(stmts[13], pyvex.IRStmt.Put)
        assert stmts[13].offset == archinfo.arch_from_id("amd64").registers["cc_dep2"][0]
        assert isinstance(stmts[13].data, pyvex.IRExpr.Const)
        assert stmts[13].data.con.value == 0
        # 14 | PUT(rip) = 0x0000000000402106
        assert isinstance(stmts[14], pyvex.IRStmt.Put)
        assert stmts[14].offset == archinfo.arch_from_id("amd64").registers["rip"][0]
        assert isinstance(stmts[14].data, pyvex.IRExpr.Const)
        assert stmts[14].data.con.value == 0x402106
        # 15 | ------ IMark(0x402106, 2, 0) ------
        assert isinstance(stmts[15], pyvex.IRStmt.IMark)
        assert stmts[15].addr == 0x402106


if __name__ == "__main__":
    unittest.main()
